<?php

namespace backend\controllers;

use Yii;
use backend\models\User;
use backend\models\SearchUser;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use backend\models\Designation;
use yii\filters\AccessControl;

/**
 * UserController implements the CRUD actions for User model.
 */
class UserController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
        'access' => [
            'class' => AccessControl::className(),
            'rules' => [
                [
                    'allow' => true,
                    'actions' => ['index','view','create','update','findModel'],
                    'roles' => ['@'],
                ],
                 [
                    'allow' => true,
                    'actions' => ['delete'],
                    'roles' => ['Client Admin'],
                ],
                    ],
                ],
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all User models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new SearchUser();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single User model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
            'id'=>$id,
            'cid'=>$this->findModel($id)->cid
        ]);
    }

    /**
     * Creates a new User model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new User();

        if ($model->load(Yii::$app->request->post()) && $model->validate()) {
            $match=Designation::findOne($model->position)->position;
            // $model->created_at=date('Y-m-d H:i:s');
            // $model->updated_at=date('Y-m-d H:i:s');

            // if($model->position_label != $match){
            //     die('Postion and Position_lable do not match');
            // }

            if($model->save())
            {
                return $this->redirect(['view', 'id' => $model->id]);
            }
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing User model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id,$cid)
    {
        // echo  $id=Yii::$app->request->post('id'); echo $cid=Yii::$app->request->post('cid'); die;
        $model = $this->findModel($id);

       
        // echo $model->id; die;
        if ($model->load(Yii::$app->request->post())) {

            // $match=Designation::findOne($model->position)->position;

            // if($model->position_label != $match){
            //     die('Postion and Position_lable do not match. Please click the back button and make sure they match.');
            // }
           
           

            if($model->save())
            {

                $auth = \Yii::$app->authManager;
                // var_dump($auth); die;
                $auth->revokeAll($id);            
                $authorRole = $auth->getRole($model->role);
                // var_dump($authorRole); die;
                $auth->assign($authorRole, $model->id);

            return $this->redirect(['view', 'id' => $model->id]);
            }
        }

        return $this->render('update', [
            'model' => $model,
            'cid'=>$cid,
        ]);
    }

    /**
     * Deletes an existing User model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the User model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return User the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = User::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
